import React, { useRef, useState } from "react";
import { Alert } from "react-native";
import {
  FlatList,
  SafeAreaView,
  StyleSheet,
  Dimensions,
  TextInput,
} from "react-native";
import { Button } from "react-native-elements";

const Item = ({ item, onDelete, actionsData, setOverlay }) => (
  <Button
    onPress={() => console.log(item)}
    onLongPress={() => {
      if (item.overlay != null) {
        Alert.alert("", "Editar accion o eliminar?", [
          {
            text: "Editar",
            onPress: () => {
              setOverlay(
                actionsData[item.idAction].overlay({
                  editOverlay: true,
                  id: item.id,
                })
              );
            },
          },
          {
            text: "Eliminar",
            onPress: () => onDelete(item.id),
          },
        ]);
      } else {
        Alert.alert("", "Eliminar accion?", [
          {
            text: "Cancelar",
          },
          {
            text: "Aceptar",
            onPress: () => onDelete(item.id),
          },
        ]);
      }
    }}
    title={item.name}
    buttonStyle={{
      borderRadius: 15,
      borderBottomRightRadius: 5,
      borderBottomLeftRadius: 5,
      padding: 8,
      paddingVertical: 4,
      margin: 5,
      backgroundColor: "#3a9bb6d6",
    }}
  />
);

const BottomList = ({
  actions,
  updateFlatList,
  onDelete,
  actionsData,
  setOverlay,
}) => {
  const [selectedId, setSelectedId] = useState(null);
  const flatList = useRef(null);
  const renderItem = ({ item }) => {
    const backgroundColor = item.id === selectedId ? "#6e3b6e" : "#f9c2ff";
    const color = item.id === selectedId ? "white" : "black";

    return (
      <Item
        actionsData={actionsData}
        item={item}
        setOverlay={setOverlay}
        onPress={() => setSelectedId(item.id)}
        backgroundColor={{ backgroundColor }}
        textColor={{ color }}
        onDelete={onDelete}
      />
    );
  };

  const scrollToEndFlatList = () => {
    setTimeout(() => {
      flatList.current.scrollToEnd(true);
    }, 50);
  };

  return (
    <SafeAreaView style={styles.container}>
      <FlatList
        inverted
        showsVerticalScrollIndicator={false}
        showsHorizontalScrollIndicator={false}
        extraData={updateFlatList}
        data={actions}
        renderItem={renderItem}
        keyExtractor={(item) => item.id}
        ref={flatList}
      />
      {scrollToEndFlatList()}
    </SafeAreaView>
  );
};

const styles = StyleSheet.create({
  container: {
    flex: 1,
    paddingHorizontal: 100,
  },
});

export default BottomList;
