import React, { useEffect, useState, useRef } from "react";
import {
  View,
  StyleSheet,
  Dimensions,
  StatusBar,
  Text,
  TouchableOpacity,
  TextInput,
  ImageBackground,
  Alert,
} from "react-native";
import AppButton from "../components/AppButton";
import TopList from "../components/TopList";
import BottomList from "../components/BottomList";
import SeparatorLine from "../components/SeparatorLine";
import defaultStyles from "../config/styles";
import Slider from "react-native-slider"; // 0.11.0
import { Button, Overlay } from "react-native-elements";

import { LogBox } from "react-native";

function MainScreen({ route, navigation }) {
  const [overlayVisible, setOverlayVisible] = useState(true);
  const [overlay, setOverlay] = useState();
  const [editOverlay, setEditOverlay] = useState();

  const input = useRef(null);
  let handleOnAccept = null;

  // action ={
  //   id: "0", ---- Id (index) on actios array
  //   idAction, ---- Id on actionsData
  //   title, ---- Action name in topList
  //   actionId, ---- Action name in backend
  //   overlay, ---- Overlay displayed when I press the action in top list or when I press to edit on Bottom List
  // }

  const actionsData = [
    {
      idAction: 0,
      title: "Avanzar",
      actionId: "Avanzar",
      overlay: ({ editOverlay = false, id }) => {
        return (
          <Overlay
            isVisible={overlayVisible}
            overlayStyle={styles.overlay}
            backdropStyle={{ height: "100%" }}
          >
            <Text style={{ marginBottom: 10 }}>
              Ingresar la cantidad de segundos
            </Text>
            <TextInput
              style={{
                borderColor: "black",
                borderWidth: 1,
                paddingHorizontal: 10,
                width: 200,
                marginBottom: 10,
              }}
              ref={input}
              onChangeText={(text) => {
                handleOnAccept = () => {
                  if (editOverlay) {
                    editAction(id, {
                      id,
                      idAction: 0,
                      name: "Avanzar " + text + " segundos",
                      actionId: "Avanzar" + "-" + text,
                      overlay: actionsData[0].overlay,
                    });
                  } else {
                    handleAddAction({
                      id: actions.length,
                      idAction: 0,
                      name: "Avanzar " + text + " segundos",
                      actionId: "Avanzar" + "-" + text,
                      overlay: actionsData[0].overlay,
                    });
                  }
                  setOverlay(false);
                };
              }}
              keyboardType="numeric"
            />
            <AppButton
              title="Aceptar"
              style={{
                width: Dimensions.get("screen").width * 0.35,
                height: Dimensions.get("screen").width * 0.1,
                borderRadius: 25,
              }}
              onPress={() => {
                if (handleOnAccept != null) handleOnAccept();
              }}
            />
          </Overlay>
        );
      },
    },
    {
      idAction: 1,
      title: "Retroceder",
      actionId: "Retroceder",
      overlay: ({ editOverlay = false, id }) => {
        return (
          <Overlay
            isVisible={overlayVisible}
            overlayStyle={styles.overlay}
            backdropStyle={{ height: "100%" }}
          >
            <Text style={{ marginBottom: 10 }}>
              Ingresar la cantidad de segundos
            </Text>
            <TextInput
              style={{
                borderColor: "black",
                borderWidth: 1,
                paddingHorizontal: 10,
                width: 200,
                marginBottom: 10,
              }}
              ref={input}
              onChangeText={(text) => {
                handleOnAccept = () => {
                  if (editOverlay) {
                    editAction(id, {
                      id,
                      idAction: 1,
                      name: "Retroceder " + text + " segundos",
                      actionId: "Retroceder" + "-" + text,
                      overlay: actionsData[1].overlay,
                    });
                  } else {
                    handleAddAction({
                      id: actions.length,
                      idAction: 1,
                      name: "Retroceder " + text + " segundos",
                      actionId: "Retroceder" + "-" + text,
                      overlay: actionsData[1].overlay,
                    });
                  }
                  setOverlay(false);
                };
              }}
              keyboardType="numeric"
            />
            <AppButton
              title="Aceptar"
              style={{
                width: Dimensions.get("screen").width * 0.35,
                height: Dimensions.get("screen").width * 0.1,
                borderRadius: 25,
              }}
              onPress={() => {
                if (handleOnAccept != null) handleOnAccept();
              }}
            />
          </Overlay>
        );
      },
    },
    {
      idAction: 2,
      title: "Izquierda",
      actionId: "Izquierda",
      overlay: ({ editOverlay = false, id }) => {
        return (
          <Overlay
            isVisible={overlayVisible}
            overlayStyle={styles.overlay}
            backdropStyle={{ height: "100%" }}
          >
            <Text style={{ marginBottom: 10 }}>
              Ingresar la cantidad de segundos
            </Text>
            <TextInput
              style={{
                borderColor: "black",
                borderWidth: 1,
                paddingHorizontal: 10,
                width: 200,
                marginBottom: 10,
              }}
              ref={input}
              onChangeText={(text) => {
                handleOnAccept = () => {
                  if (editOverlay) {
                    editAction(id, {
                      id,
                      idAction: 2,
                      name: "Izquierda " + text + " segundos",
                      actionId: "Izquierda" + "-" + text,
                      overlay: actionsData[2].overlay,
                    });
                  } else {
                    handleAddAction({
                      id: actions.length,
                      idAction: 2,
                      name: "Izquierda " + text + " segundos",
                      actionId: "Izquierda" + "-" + text,
                      overlay: actionsData[2].overlay,
                    });
                  }
                  setOverlay(false);
                };
              }}
              keyboardType="numeric"
            />
            <AppButton
              title="Aceptar"
              style={{
                width: Dimensions.get("screen").width * 0.35,
                height: Dimensions.get("screen").width * 0.1,
                borderRadius: 25,
              }}
              onPress={() => {
                if (handleOnAccept != null) handleOnAccept();
              }}
            />
          </Overlay>
        );
      },
    },
    {
      idAction: 3,
      title: "Derecha",
      actionId: "Derecha",
      overlay: ({ editOverlay = false, id }) => {
        return (
          <Overlay
            isVisible={overlayVisible}
            overlayStyle={styles.overlay}
            backdropStyle={{ height: "100%" }}
          >
            <Text style={{ marginBottom: 10 }}>
              Ingresar la cantidad de segundos
            </Text>
            <TextInput
              style={{
                borderColor: "black",
                borderWidth: 1,
                paddingHorizontal: 10,
                width: 200,
                marginBottom: 10,
              }}
              ref={input}
              onChangeText={(text) => {
                handleOnAccept = () => {
                  if (editOverlay) {
                    editAction(id, {
                      id,
                      idAction: 3,
                      name: "Derecha " + text + " segundos",
                      actionId: "Derecha" + "-" + text,
                      overlay: actionsData[3].overlay,
                    });
                  } else {
                    handleAddAction({
                      id: actions.length,
                      idAction: 3,
                      name: "Derecha " + text + " segundos",
                      actionId: "Derecha" + "-" + text,
                      overlay: actionsData[3].overlay,
                    });
                  }
                  setOverlay(false);
                };
              }}
              keyboardType="numeric"
            />
            <AppButton
              title="Aceptar"
              style={{
                width: Dimensions.get("screen").width * 0.35,
                height: Dimensions.get("screen").width * 0.1,
                borderRadius: 25,
              }}
              onPress={() => {
                if (handleOnAccept != null) handleOnAccept();
              }}
            />
          </Overlay>
        );
      },
    },
    {
      idAction: 4,
      title: "Avanzar hasta ver color",
      actionId: "AvanzarHastaVerColor",
      overlay: ({ editOverlay = false, id }) => {
        return (
          <Overlay
            isVisible={overlayVisible}
            overlayStyle={styles.overlay}
            backdropStyle={{ height: "100%" }}
          >
            <Text
              style={{
                marginBottom: "5%",
                fontSize: Dimensions.get("screen").width * 0.035,
              }}
            >
              Seleccionar un color
            </Text>
            <View style={{ flexDirection: "row" }}>
              <TouchableOpacity
                onPress={() => {
                  if (editOverlay) {
                    editAction(id, {
                      id: id,
                      idAction: 4,
                      name: "Avanzar hasta ver blanco",
                      actionId: "AvanzarHastaVerColor-blanco",
                      overlay: actionsData[3].overlay,
                    });
                  } else {
                    handleAddAction({
                      id: actions.length,
                      idAction: 4,
                      name: "Avanzar hasta ver blanco",
                      actionId: "AvanzarHastaVerColor-blanco",
                      overlay: actionsData[4].overlay,
                    });
                  }
                  setOverlay(false);
                }}
              >
                <View
                  style={{
                    height: 60,
                    width: 60,
                    borderWidth: 5,
                    borderColor: "black",
                    margin: "3%",
                  }}
                />
              </TouchableOpacity>
              <TouchableOpacity
                onPress={() => {
                  if (editOverlay) {
                    editAction(id, {
                      id: id,
                      idAction: 4,
                      name: "Avanzar hasta ver negro",
                      actionId: "AvanzarHastaVerColor-negro",
                      overlay: actionsData[3].overlay,
                    });
                  } else {
                    handleAddAction({
                      id: actions.length,
                      idAction: 4,
                      name: "Avanzar hasta ver negro",
                      actionId: "AvanzarHastaVerColor-negro",
                      overlay: actionsData[4].overlay,
                    });
                  }
                  setOverlay(false);
                }}
              >
                <View
                  style={{
                    height: 60,
                    width: 60,
                    borderWidth: 5,
                    borderColor: "black",
                    backgroundColor: "black",
                    margin: "3%",
                  }}
                />
              </TouchableOpacity>
            </View>
          </Overlay>
        );
      },
    },

    {
      idAction: 5,
      title: "Avanzar hasta obstaculo",
      actionId: "AvanzarHastaObstaculo",
      overlay: null,
    },
    {
      idAction: 6,
      title: "Accion 1",
      actionId: "Accion1",
      overlay: null,
    },
  ];

  LogBox.ignoreAllLogs();
  const { port, ipServer } = route.params;
  const [running, setRunning] = useState(false);

  const [actions, setActions] = useState([]);
  const [updateFlatList, setUpdateFlatList] = useState(false);
  const [sliderValue, setSliderValue] = useState(400);

  const handleAddAction = (data) => {
    actions.push(data);
    setActions(actions);
    setUpdateFlatList(!updateFlatList);
  };

  const deleteAction = (index) => {
    actions.splice(index, 1);
    setUpdateFlatList(!updateFlatList);
  };

  const updateBottomList = () => {
    setUpdateFlatList(!updateFlatList);
  };
  const editAction = (id, data) => {
    actions.splice(id, 1, data);
    setUpdateFlatList(!updateFlatList);
  };

  const handleClear = () => {
    setActions([]);
    setUpdateFlatList(!updateFlatList);
  };

  const sendData = async (message) => {
    try {
      let socket = new WebSocket("ws://" + ipServer + ":" + port);
      socket.binaryType = "blob";

      socket.onopen = () => {
        socket.send(message);
      };

      socket.onmessage = ({ data }) => {
        setRunning(false);
      };
    } catch (error) {
      Alert.alert(
        "",
        "Se ha producido un arror, verificar si el servidor esta activo, o la direccion IP y el puerto sonb correctos"
      );
    }
  };

  return (
    <ImageBackground
      blurRadius={0.7}
      source={require("../assets/background2.jpg")}
      style={{
        flex: 1,
        padding: 10,
        paddingTop: StatusBar.currentHeight,
      }}
    >
      <View style={{ paddingTop: "1%" }}>
        <Text
          style={{
            color: "white",
            textAlign: "center",
            fontSize: 14,
          }}
        >
          Para agregarle una acción al robot, seleccionarla desde el panel
          superior. Para deshacer o editar una, mantenen pulsada en el panel
          inferior la accion que desees.{" "}
        </Text>
      </View>
      <View style={styles.topView}>
        <TopList
          showOverlay={(overlay) => setOverlay(overlay)}
          actions={actions}
          actionsData={actionsData}
          handleAddAction={handleAddAction}
          overlay2={overlay}
        />
      </View>
      <SeparatorLine />
      <View
        style={{
          flexDirection: "row",
          justifyContent: "space-evenly",
          alignItems: "center",
        }}
      >
        <AppButton
          color={defaultStyles.grey}
          title="Eliminar todas las acciones"
          style={styles.buttonStyle}
          onPress={handleClear}
        />
        <AppButton
          color={defaultStyles.grey}
          title="Configuracion"
          style={styles.buttonStyle}
          onPress={() => navigation.navigate("ConfigScreen")}
        />
      </View>
      <View
        style={{
          padding: "3%",
        }}
      >
        <Slider
          minimumTrackTintColor={"white"}
          thumbTintColor={"white"}
          value={sliderValue}
          minimumValue={200}
          maximumValue={1024}
          onValueChange={(value) => setSliderValue(Math.floor(value))}
        />
        <Text
          style={{ textAlign: "center", color: "white", fontWeight: "bold" }}
        >
          Velocidad del robot: {sliderValue}
        </Text>
      </View>
      <SeparatorLine />
      <View style={styles.bottomView}>
        <BottomList
          actionsData={actionsData}
          setOverlay={(param) => setOverlay(param)}
          actions={actions}
          addAction={setActions}
          updateFlatList={updateFlatList}
          onDelete={deleteAction}
        />
        {!false ? (
          <AppButton
            color="#20bc20"
            title="Empezar"
            style={{
              width: Dimensions.get("screen").width * 0.35,
              height: Dimensions.get("screen").width * 0.1,
              borderRadius: 25,
            }}
            onPress={() => {
              if (actions.length == 0) {
                Alert.alert("", "Debe seleccionar por lo menos una accion");
                return;
              }
              setRunning(true);

              let dataToSend = "";
              actions.forEach((item) => (dataToSend += item.actionId + "\n"));
              dataToSend += sliderValue;
              sendData(dataToSend);
            }}
          />
        ) : (
          <View style={{ height: Dimensions.get("screen").width * 0.17 }} />
        )}
      </View>
      {overlay}
      {editOverlay}
    </ImageBackground>
  );
}

const styles = StyleSheet.create({
  topView: {
    marginTop: "1%",
    alignItems: "center",
    alignContent: "flex-start",
    flexDirection: "row",
    height: "17%",
    paddingBottom: 12,
  },
  bottomView: {
    flex: 1,
    marginBottom: "2%",
    alignItems: "center",
    justifyContent: "center",
  },
  linearGradient: {
    flex: 1,
  },
  buttonStyle: {
    borderRadius: 10,
    padding: 5,
  },
  overlay: {
    alignItems: "center",
    justifyContent: "space-around",
    width: "80%",
    borderRadius: 10,
    paddingVertical: "8%",
  },
});

export default MainScreen;
